/**
 * 
 */
package org.gcube.portlets.user.gcubegeoexplorer.server;



import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.gcube.application.framework.core.session.ASLSession;
import org.gcube.application.framework.core.session.SessionManager;
import org.gcube.common.core.contexts.GHNContext;
import org.gcube.common.core.informationsystem.client.AtomicCondition;
import org.gcube.common.core.informationsystem.client.ISClient;
import org.gcube.common.core.informationsystem.client.queries.GCUBERuntimeResourceQuery;
import org.gcube.common.core.resources.GCUBERuntimeResource;
import org.gcube.common.core.resources.runtime.AccessPoint;
import org.gcube.common.core.scope.GCUBEScope;
import org.gcube.common.core.scope.GCUBEScope.MalformedScopeExpressionException;
import org.gcube.common.core.utils.logging.GCUBEClientLog;
import org.gcube.common.scope.api.ScopeProvider;
import org.gcube.portlets.user.geoexplorer.server.GeoExplorerServiceImpl;
import org.gcube.portlets.user.geoexplorer.server.GeoExplorerServiceInterface;
import org.gcube.portlets.user.geoexplorer.server.GeoExplorerServiceParameters;

/**
 * @author "Federico De Faveri defaveri@isti.cnr.it"
 * @author Modified By Francesco Mangiacrapa
 *
 */
public class GCubeGeoExplorerServletImpl  extends GeoExplorerServiceImpl {

	private static final long serialVersionUID = 804152795418658243L;

	public static final String USERNAME_ATTRIBUTE = "username";
	
	public static final String GEOSERVER_RESOURCE_NAME = "GeoServer";
	public static final String GEONETWORK_RESOURCE_NAME = "GeoNetwork";
	
	public static final String GEOCALLER_ATTRIBUTE_NAME = "GEOCALLER";
	private static final String WORKSPACES_PROPERTY_NAME = "workspaces";
	public static final long CACHE_REFRESH_TIME = 10*60*1000;

	public static GCUBEClientLog log = new GCUBEClientLog(GCubeGeoExplorerServletImpl.class);

	protected Map<GCUBEScope, GeoExplorerServiceInterface> parametersCache = new HashMap<GCUBEScope, GeoExplorerServiceInterface>();
	protected Logger logger = Logger.getLogger(GCubeGeoExplorerServletImpl.class);
	protected Timer timer;

	protected List<String> listUrlInternalGeoserver = new ArrayList<String>();
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init() throws ServletException {
//		super.init();
		timer = new Timer(true);
//		timer.schedule(new TimerTask() {
//			@Override
//			public void run() {
//				refreshParametersCache();
//			}
//		}, 0, CACHE_REFRESH_TIME);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void destroy() {
		super.destroy();
		timer.cancel();
	}

	protected ASLSession getASLSession(HttpSession httpSession)
	{
		String sessionID = httpSession.getId();
		String user = (String) httpSession.getAttribute(USERNAME_ATTRIBUTE);

		if (user == null) {

			logger.trace("GCubeGeoExplorerServletImpl STARTING IN TEST MODE - NO USER FOUND");

			//for test only
			user = "test.user";
			httpSession.setAttribute(USERNAME_ATTRIBUTE, user);
			
			
			ASLSession session = SessionManager.getInstance().getASLSession(sessionID, user);
//			String defaultScope = "/d4science.research-infrastructures.eu/gCubeApps/EcologicalModelling"; //PRODUCTION
			String defaultScope = "/gcube/devsec/devVRE"; //DEVELOPMENT
			session.setScope(defaultScope);
			//session.setScope("/gcube/devsec/devVRE");

			return session;
		} else logger.trace("user found in session "+user);
		return SessionManager.getInstance().getASLSession(sessionID, user);
	}


	protected ServerParameters getParameters(GCUBEScope scope, boolean isGeoserver) throws Exception
	{
		
		ServerParameters parameters = new ServerParameters();
		
		try{

			String resourceName = (isGeoserver ? GEOSERVER_RESOURCE_NAME : GEONETWORK_RESOURCE_NAME); 


			logger.trace("instance ISClient ");
			ISClient client = GHNContext.getImplementation(ISClient.class);

			logger.trace("instance rtrQuery ");
			GCUBERuntimeResourceQuery rtrQuery = client.getQuery(GCUBERuntimeResourceQuery.class);

			rtrQuery.addAtomicConditions(new AtomicCondition("/Profile/Name",resourceName));
			logger.trace("execute rtrQuery ");
			List<GCUBERuntimeResource> rtrs = client.execute(rtrQuery, scope);

			if (rtrs.size()==0) throw new Exception("Resource "+resourceName+" not found");
	
			GCUBERuntimeResource rtr = rtrs.get(0);
	
			if (rtr.getAccessPoints().size()==0) throw new Exception("Accesspoint in resource "+resourceName+" not found");
	
			AccessPoint ap = rtr.getAccessPoints().get(0);
			parameters.setUrl(ap.getEndpoint());
			parameters.setUser(ap.getUsername()); //username
			parameters.setPassword(ap.getPassword()); //password
	
			if (isGeoserver)
				parameters.setWorkspaces(ap.getProperty(WORKSPACES_PROPERTY_NAME));

			
			
			/*
			SimpleQuery query = queryFor(ServiceEndpoint.class);
			 
			query.addCondition("$resource/Profile/Category/text() eq 'Gis'")
			.addCondition("$resource/Profile/Platform/Name/text() eq '"+resourceName+"'")
			.setResult("$resource/Profile/AccessPoint");
			 
			DiscoveryClient<AccessPoint> client = clientFor(AccessPoint.class);
			 
			List<AccessPoint> accesspoints = client.submit(query);
			
			
			if (accesspoints.size()==0) throw new Exception("Accesspoint in resource "+resourceName+" not found");

			
			AccessPoint ap = accesspoints.get(0);
			parameters.setUrl(ap.address());
			parameters.setUser(ap.username()); //username
			parameters.setPassword(ap.password()); //password

			if (isGeoserver)
				parameters.setWorkspaces("");
			
			*/

			
		}catch (Exception e) {
			logger.error("Sorry, an error occurred on reading parameters in Runtime Reosurces",e);
			e.printStackTrace();
		}

		return parameters;
	}

	protected GeoExplorerServiceInterface retrieveGisParameters(GCUBEScope scope) throws Exception
	{
		GeoExplorerServiceParameters gisViewerServiceParameters = new GeoExplorerServiceParameters();
		gisViewerServiceParameters.setScope(scope.getName());

		try {
			ServerParameters geoServerParameters = getParameters(scope, true);
			gisViewerServiceParameters.setGeoServerUrl(geoServerParameters.getUrl());
			gisViewerServiceParameters.setGeoServerUser(geoServerParameters.getUser());
			gisViewerServiceParameters.setGeoServerPwd(geoServerParameters.getPassword());
//			gisViewerServiceParameters.setWorkspaces(geoServerParameters.getWorkspaces());
			
			listUrlInternalGeoserver.add(geoServerParameters.getUrl());
			
		} catch (Exception e)
		{
			logger.error("Error retrieving the GeoServer parameters", e);
			throw new Exception("Error retrieving the GeoServer parameters", e);
		}

		try {
			ServerParameters geoNetworkParameters = getParameters(scope, false);
			gisViewerServiceParameters.setGeoNetworkUrl(geoNetworkParameters.getUrl());
			gisViewerServiceParameters.setGeoNetworkUser(geoNetworkParameters.getUser());
			gisViewerServiceParameters.setGeoNetworkPwd(geoNetworkParameters.getPassword());

		} catch (Exception e)
		{
			logger.error("Error retrieving the Gis parameters", e);
			throw new Exception("Error retrieving the GeoNetwork parameters", e);
		}
		
		logger.trace("retrieved parameters: "+gisViewerServiceParameters);

		return gisViewerServiceParameters;		
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected GeoExplorerServiceInterface getGeoParameters() throws Exception {
		//httpSession = this.getThreadLocalRequest().getSession();
		GeoExplorerServiceInterface parameters = null;
		try{
			
			ASLSession session = getASLSession(this.getThreadLocalRequest().getSession());
			GCUBEScope scope = session.getScope();
	
			parameters = parametersCache.get(scope);
	
			if (parameters == null) {
				parameters = retrieveGisParameters(scope);
				parametersCache.put(scope, parameters);
			}
			
		}catch (Exception e) {
			logger.error("Error retrieving Geo Parameters", e);
		}

		return parameters;
	}

	protected void refreshParametersCache()
	{
		for (GCUBEScope scope:parametersCache.keySet()) {
			try{
				GeoExplorerServiceInterface parameters = retrieveGisParameters(scope);
				parametersCache.put(scope, parameters);
			} catch (Exception e) {
				logger.warn("An error occured retrieving gis parameters for scope "+scope, e);
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.geoexplorer.server.GeoExplorerServiceImpl#getInternalGeoserver()
	 */
	@Override
	protected List<String> getInternalGeoserver() throws Exception {
		return listUrlInternalGeoserver;
	}
	
	public static void main(String[] args) throws MalformedScopeExpressionException, Exception
	{
		
		try{
			String DEFAULT_SCOPE = "/gcube/devsec/devVRE";
			ScopeProvider.instance.set(DEFAULT_SCOPE);
			
			GCubeGeoExplorerServletImpl impl = new GCubeGeoExplorerServletImpl();
			GeoExplorerServiceInterface parameters = impl.retrieveGisParameters(GCUBEScope.getScope(DEFAULT_SCOPE));
			
			
			System.out.println(parameters);
		}catch (Exception e) {
			e.printStackTrace();
		}

	}
	
}
